import * as vue_demi from 'vue-demi';
import * as redux from 'redux';
import { Store, Dispatch, AnyAction } from 'redux';
import * as _reduxjs_toolkit_dist_query from '@reduxjs/toolkit/dist/query';
import * as _reduxjs_toolkit_dist_query_core_module from '@reduxjs/toolkit/dist/query/core/module';
import { MutationDefinition, QueryArgFrom, QueryDefinition, EndpointDefinitions } from '@reduxjs/toolkit/dist/query/endpointDefinitions';
import { SkipToken, QueryStatus, BaseQueryFn } from '@reduxjs/toolkit/query';
import { MutationActionCreatorResult, QueryActionCreatorResult } from '@reduxjs/toolkit/dist/query/core/buildInitiate';
import { MutationResultSelectorResult } from '@reduxjs/toolkit/dist/query/core/buildSelectors';
import { NoInfer, Id, Override } from '@reduxjs/toolkit/dist/query/tsHelpers';
import { QuerySubState, SubscriptionOptions } from '@reduxjs/toolkit/dist/query/core/apiState';
import { Ref } from 'vue';

declare const ReduxStore: unique symbol;

declare type Selector<S, R> = (state: S) => R;
declare const useSelector: <S = any, R = any>(select: Selector<S, R>) => vue_demi.ComputedRef<R>;

declare const useStore: <S extends Store<any, redux.AnyAction> = Store<any, redux.AnyAction>>() => S;

declare const useDispatch: <D extends Dispatch<AnyAction> = Dispatch<AnyAction>>() => D;

declare type UseMutationStateResult<_ extends MutationDefinition<any, any, any, any>, R> = NoInfer<R>;
declare type UseMutation<D extends MutationDefinition<any, any, any, any>> = <R extends Record<string, any> = MutationResultSelectorResult<D>>() => [
    (arg: QueryArgFrom<D>) => MutationActionCreatorResult<D>,
    UseMutationStateResult<D, R>
];

declare type Reactive<T> = Ref<T> | T;
declare type ReactiveRecord<T> = {
    [K in keyof T]: Reactive<T[K]>;
};

declare type UseQueryStateBaseResult<D extends QueryDefinition<any, any, any, any>> = QuerySubState<D> & {
    /**
     * Query has not started yet.
     */
    isUninitialized: false;
    /**
     * Query is currently loading for the first time. No data yet.
     */
    isLoading: false;
    /**
     * Query is currently fetching, but might have data from an earlier request.
     */
    isFetching: false;
    /**
     * Query has data from a successful load.
     */
    isSuccess: false;
    /**
     * Query is currently in "error" state.
     */
    isError: false;
};
declare type UseQueryStateDefaultResult<D extends QueryDefinition<any, any, any, any>> = Id<Override<Extract<UseQueryStateBaseResult<D>, {
    status: QueryStatus.uninitialized;
}>, {
    isUninitialized: true;
}> | Override<UseQueryStateBaseResult<D>, {
    isLoading: true;
    isFetching: boolean;
    data: undefined;
} | ({
    isSuccess: true;
    isFetching: boolean;
    error: undefined;
} & Required<Pick<UseQueryStateBaseResult<D>, 'data' | 'fulfilledTimeStamp'>>) | ({
    isError: true;
} & Required<Pick<UseQueryStateBaseResult<D>, 'error'>>)>>;
declare type UseQueryStateOptions<D extends QueryDefinition<any, any, any, any>, R extends Record<string, any>> = {
    /**
     * Prevents a query from automatically running.
     *
     * @remarks
     * When skip is true:
     *
     * - **If the query has cached data:**
     *   * The cached data **will not be used** on the initial load, and will ignore updates from any identical query until the `skip` condition is removed
     *   * The query will have a status of `uninitialized`
     *   * If `skip: false` is set after skipping the initial load, the cached result will be used
     * - **If the query does not have cached data:**
     *   * The query will have a status of `uninitialized`
     *   * The query will not exist in the state when viewed with the dev tools
     *   * The query will not automatically fetch on mount
     *   * The query will not automatically run when additional components with the same query are added that do run
     *
     * @example
     * ```ts
     * // codeblock-meta title="Skip example"
     * const Pokemon = ({ name, skip }: { name: string; skip: boolean }) => {
     *   const { data, error, status } = useGetPokemonByNameQuery(name, {
     *     skip,
     *   });
     *
     *   return (
     *     <div>
     *       {name} - {status}
     *     </div>
     *   );
     * };
     * ```
     */
    skip?: boolean;
};
declare type UseQueryState<D extends QueryDefinition<any, any, any, any>> = <R = UseQueryStateDefaultResult<D>>(arg: Reactive<QueryArgFrom<D> | SkipToken>, options?: ReactiveRecord<UseQueryStateOptions<D, R>>) => UseQueryStateResult<D, R>;
declare type UseQueryStateResult<_ extends QueryDefinition<any, any, any, any>, R> = NoInfer<R>;

interface UseQuerySubscriptionOptions extends SubscriptionOptions {
    /**
     * Prevents a query from automatically running.
     *
     * @remarks
     * When `skip` is true (or `skipToken` is passed in as `arg`):
     *
     * - **If the query has cached data:**
     *   * The cached data **will not be used** on the initial load, and will ignore updates from any identical query until the `skip` condition is removed
     *   * The query will have a status of `uninitialized`
     *   * If `skip: false` is set after skipping the initial load, the cached result will be used
     * - **If the query does not have cached data:**
     *   * The query will have a status of `uninitialized`
     *   * The query will not exist in the state when viewed with the dev tools
     *   * The query will not automatically fetch on mount
     *   * The query will not automatically run when additional components with the same query are added that do run
     *
     * @example
     * ```tsx
     * // codeblock-meta title="Skip example"
     * const Pokemon = ({ name, skip }: { name: string; skip: boolean }) => {
     *   const { data, error, status } = useGetPokemonByNameQuery(name, {
     *     skip,
     *   });
     *
     *   return (
     *     <div>
     *       {name} - {status}
     *     </div>
     *   );
     * };
     * ```
     */
    skip?: boolean;
    /**
     * Defaults to `false`. This setting allows you to control whether if a cached result is already available, RTK Query will only serve a cached result, or if it should `refetch` when set to `true` or if an adequate amount of time has passed since the last successful query result.
     * - `false` - Will not cause a query to be performed _unless_ it does not exist yet.
     * - `true` - Will always refetch when a new subscriber to a query is added. Behaves the same as calling the `refetch` callback or passing `forceRefetch: true` in the action creator.
     * - `number` - **Value is in seconds**. If a number is provided and there is an existing query in the cache, it will compare the current time vs the last fulfilled timestamp, and only refetch if enough time has elapsed.
     *
     * If you specify this option alongside `skip: true`, this **will not be evaluated** until `skip` is false.
     */
    refetchOnMountOrArgChange?: boolean | number;
}
declare type UseQuerySubscription<D extends QueryDefinition<any, any, any, any>> = (arg: Reactive<QueryArgFrom<D> | SkipToken>, options?: ReactiveRecord<UseQuerySubscriptionOptions>) => Pick<QueryActionCreatorResult<D>, 'refetch'>;

declare type UseQuery<D extends QueryDefinition<any, any, any, any>> = <R extends Record<string, any> = UseQueryStateDefaultResult<D>>(arg: Reactive<QueryArgFrom<D> | SkipToken>, options?: ReactiveRecord<UseQuerySubscriptionOptions & UseQueryStateOptions<D, R>>) => UseQueryStateResult<D, R> & ReturnType<UseQuerySubscription<D>>;

declare enum DefinitionType {
    query = "query",
    mutation = "mutation"
}
declare const vueHooksModuleName: unique symbol;
declare type QueryHooks<D extends QueryDefinition<any, any, any, any, any>> = {
    useQuery: UseQuery<D>;
    useQuerySubscription: UseQuerySubscription<D>;
    useQueryState: UseQueryState<D>;
};
declare type MutationHooks<D extends MutationDefinition<any, any, any, any, any>> = {
    useMutation: UseMutation<D>;
};
declare type HooksWithUniqueNames<Definitions extends EndpointDefinitions> = keyof Definitions extends infer Keys ? Keys extends string ? Definitions[Keys] extends {
    type: DefinitionType.query;
} ? {
    [K in Keys as `use${Capitalize<K>}Query`]: UseQuery<Extract<Definitions[K], QueryDefinition<any, any, any, any>>>;
} : Definitions[Keys] extends {
    type: DefinitionType.mutation;
} ? {
    [K in Keys as `use${Capitalize<K>}Mutation`]: UseMutation<Extract<Definitions[K], MutationDefinition<any, any, any, any>>>;
} : never : never : never;
declare module '@reduxjs/toolkit/dist/query/apiTypes' {
    interface ApiModules<BaseQuery extends BaseQueryFn, Definitions extends EndpointDefinitions, ReducerPath extends string, TagTypes extends string> {
        [vueHooksModuleName]: {
            /**
             *  Endpoints based on the input endpoints provided to `createApi`, containing `select`, `hooks` and `action matchers`.
             */
            endpoints: {
                [K in keyof Definitions]: Definitions[K] extends QueryDefinition<any, any, any, any, any> ? QueryHooks<Definitions[K]> : Definitions[K] extends MutationDefinition<any, any, any, any, any> ? MutationHooks<Definitions[K]> : never;
            };
        } & HooksWithUniqueNames<Definitions>;
    }
}
declare const createApi: _reduxjs_toolkit_dist_query.CreateApi<typeof _reduxjs_toolkit_dist_query_core_module.coreModuleName | typeof vueHooksModuleName>;

export { ReduxStore, createApi, useDispatch, useSelector, useStore };
